@@ -1,5 +1,6 @@
 #include "rar.hpp"
 
+
 static bool UserBreak;
 
 ErrorHandler::ErrorHandler()
@@ -42,6 +43,8 @@ void ErrorHandler::CloseError(const char *FileName)
     ErrMsg(NULL,St(MErrFClose),FileName);
     SysErrMsg();
   }
+#endif
+#if !defined(SILENT) || defined(RARDLL)
   Throw(FATAL_ERROR);
 #endif
 }
@@ -50,7 +53,9 @@ void ErrorHandler::CloseError(const char *FileName)
 void ErrorHandler::ReadError(const char *FileName)
 {
 #ifndef SILENT
-  ReadErrorMsg(FileName);
+  ReadErrorMsg(NULL,FileName);
+#endif
+#if !defined(SILENT) || defined(RARDLL)
   Throw(FATAL_ERROR);
 #endif
 }
@@ -58,9 +63,10 @@ void ErrorHandler::ReadError(const char *FileName)
 
 bool ErrorHandler::AskRepeatRead(const char *FileName)
 {
-#if !defined(SILENT) && !defined(SFX_MODULE)
+#if !defined(SILENT) && !defined(SFX_MODULE) && !defined(_WIN_CE)
   if (!Silent)
   {
+    SysErrMsg();
     mprintf("\n");
     Log(NULL,St(MErrRead),FileName);
     return(Ask(St(MRetryAbort))==1);
@@ -70,11 +76,12 @@ bool ErrorHandler::AskRepeatRead(const char *FileName)
 }
 
 
-void ErrorHandler::WriteError(const char *FileName)
+void ErrorHandler::WriteError(const char *ArcName,const char *FileName)
 {
 #ifndef SILENT
-  ErrMsg(NULL,St(MErrWrite),FileName);
-  SysErrMsg();
+  WriteErrorMsg(ArcName,FileName);
+#endif
+#if !defined(SILENT) || defined(RARDLL)
   Throw(WRITE_ERROR);
 #endif
 }
@@ -86,19 +93,22 @@ void ErrorHandler::WriteErrorFAT(const char *FileName)
 #if !defined(SILENT) && !defined(SFX_MODULE)
   SysErrMsg();
   ErrMsg(NULL,St(MNTFSRequired),FileName);
+#endif
+#if !defined(SILENT) && !defined(SFX_MODULE) || defined(RARDLL)
   Throw(WRITE_ERROR);
 #endif
 }
 #endif
 
 
-bool ErrorHandler::AskRepeatWrite(const char *FileName)
+bool ErrorHandler::AskRepeatWrite(const char *FileName,bool DiskFull)
 {
-#ifndef SILENT
+#if !defined(SILENT) && !defined(_WIN_CE)
   if (!Silent)
   {
+    SysErrMsg();
     mprintf("\n");
-    Log(NULL,St(MErrWrite),FileName);
+    Log(NULL,St(DiskFull ? MNotEnoughDisk:MErrWrite),FileName);
     return(Ask(St(MRetryAbort))==1);
   }
 #endif
@@ -114,11 +124,22 @@ void ErrorHandler::SeekError(const char *FileName)
     ErrMsg(NULL,St(MErrSeek),FileName);
     SysErrMsg();
   }
+#endif
+#if !defined(SILENT) || defined(RARDLL)
   Throw(FATAL_ERROR);
 #endif
 }
 
 
+void ErrorHandler::GeneralErrMsg(const char *Msg)
+{
+#ifndef SILENT
+  Log(NULL,"%s",Msg);
+  SysErrMsg();
+#endif
+}
+
+
 void ErrorHandler::MemoryErrorMsg()
 {
 #ifndef SILENT
@@ -128,9 +149,15 @@ void ErrorHandler::MemoryErrorMsg()
 
 
 void ErrorHandler::OpenErrorMsg(const char *FileName)
+{
+  OpenErrorMsg(NULL,FileName);
+}
+
+
+void ErrorHandler::OpenErrorMsg(const char *ArcName,const char *FileName)
 {
 #ifndef SILENT
-  Log(NULL,St(MCannotOpen),FileName);
+  Log(ArcName && *ArcName ? ArcName:NULL,St(MCannotOpen),FileName);
   Alarm();
   SysErrMsg();
 #endif
@@ -138,19 +165,50 @@ void ErrorHandler::OpenErrorMsg(const char *FileName)
 
 
 void ErrorHandler::CreateErrorMsg(const char *FileName)
+{
+  CreateErrorMsg(NULL,FileName);
+}
+
+
+void ErrorHandler::CreateErrorMsg(const char *ArcName,const char *FileName)
 {
 #ifndef SILENT
-  Log(NULL,St(MCannotCreate),FileName);
+  Log(ArcName && *ArcName ? ArcName:NULL,St(MCannotCreate),FileName);
   Alarm();
+#if defined(_WIN_32) && !defined(_WIN_CE) && !defined(SFX_MODULE) && defined(MAX_PATH)
+  if (GetLastError()==ERROR_PATH_NOT_FOUND)
+  {
+    size_t NameLength=strlen(FileName);
+    if (!IsFullPath(FileName))
+    {
+      char CurDir[NM];
+      GetCurrentDirectory(sizeof(CurDir),CurDir);
+      NameLength+=strlen(CurDir)+1;
+    }
+    if (NameLength>MAX_PATH)
+    {
+      Log(ArcName && *ArcName ? ArcName:NULL,St(MMaxPathLimit),MAX_PATH);
+    }
+  }
+#endif
   SysErrMsg();
 #endif
 }
 
 
-void ErrorHandler::ReadErrorMsg(const char *FileName)
+void ErrorHandler::ReadErrorMsg(const char *ArcName,const char *FileName)
 {
 #ifndef SILENT
-  ErrMsg(NULL,St(MErrRead),FileName);
+  ErrMsg(ArcName,St(MErrRead),FileName);
+  SysErrMsg();
+#endif
+}
+
+
+void ErrorHandler::WriteErrorMsg(const char *ArcName,const char *FileName)
+{
+#ifndef SILENT
+  ErrMsg(ArcName,St(MErrWrite),FileName);
   SysErrMsg();
 #endif
 }
@@ -166,7 +224,7 @@ void ErrorHandler::Exit(int ExitCode)
 
 
 #ifndef GUI
-void ErrorHandler::ErrMsg(char *ArcName,const char *fmt,...)
+void ErrorHandler::ErrMsg(const char *ArcName,const char *fmt,...)
 {
   safebuf char Msg[NM+1024];
   va_list argptr;
@@ -224,12 +282,18 @@ void _stdfunction ProcessSignal(int SigType)
 #endif
   UserBreak=true;
   mprintf(St(MBreak));
-  File::RemoveCreated();
-#if defined(USE_RC) && !defined(SFX_MODULE)
+  for (int I=0;!File::RemoveCreated() && I<3;I++)
+  {
+#ifdef _WIN_32
+    Sleep(100);
+#endif
+  }
+#if defined(USE_RC) && !defined(SFX_MODULE) && !defined(_WIN_CE)
   ExtRes.UnloadDLL();
 #endif
   exit(USER_BREAK);
-#ifdef _WIN_32
+#if defined(_WIN_32) && !defined(_MSC_VER)
+  // never reached, just to avoid a compiler warning
   return(TRUE);
 #endif
 }
@@ -267,23 +331,26 @@ void ErrorHandler::Throw(int Code)
 
 void ErrorHandler::SysErrMsg()
 {
-#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SILENT)
-  char *lpMsgBuf=NULL;
+#if !defined(SFX_MODULE) && !defined(SILENT)
+#ifdef _WIN_32
+    #define STRCHR strchr
+    #define ERRCHAR char
+  ERRCHAR  *lpMsgBuf=NULL;
   int ErrType=GetLastError();
   if (ErrType!=0 && FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
-              NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-              (LPTSTR) &lpMsgBuf,0,NULL))
+              NULL,ErrType,MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+              (LPTSTR)&lpMsgBuf,0,NULL))
   {
-    char *CurMsg=lpMsgBuf;
+    ERRCHAR  *CurMsg=lpMsgBuf;
     while (CurMsg!=NULL)
     {
       while (*CurMsg=='\r' || *CurMsg=='\n')
         CurMsg++;
       if (*CurMsg==0)
         break;
-      char *EndMsg=strchr(CurMsg,'\r');
+      ERRCHAR *EndMsg=STRCHR(CurMsg,'\r');
       if (EndMsg==NULL)
-        EndMsg=strchr(CurMsg,'\n');
+        EndMsg=STRCHR(CurMsg,'\n');
       if (EndMsg!=NULL)
       {
         *EndMsg=0;
@@ -295,4 +362,16 @@ void ErrorHandler::SysErrMsg()
   }
   LocalFree( lpMsgBuf );
 #endif
+
+#if defined(_UNIX) || defined(_EMX)
+  char *err=strerror(errno);
+  if (err!=NULL)
+    Log(NULL,"\n%s",err);
+#endif
+
+#endif
 }
+
+
+
+
