@@ -43,6 +43,12 @@ void Unpack::Init(byte *Window)
     ExternalWindow=true;
   }
   UnpInitData(false);
+
+#ifndef SFX_MODULE
+  // RAR 1.5 decompression initialization
+  OldUnpInitData(false);
+  InitHuff();
+#endif
 }
 
 
@@ -51,15 +57,16 @@ void Unpack::DoUnpack(int Method,bool Solid)
   switch(Method)
   {
 #ifndef SFX_MODULE
-    case 15:
+    case 15: // rar 1.5 compression
       Unpack15(Solid);
       break;
-    case 20:
-    case 26:
+    case 20: // rar 2.x compression
+    case 26: // files larger than 2GB
       Unpack20(Solid);
       break;
 #endif
-    case 29:
+    case 29: // rar 3.x compression
+    case 36: // alternative hash
       Unpack29(Solid);
       break;
   }
@@ -189,9 +196,6 @@ void Unpack::Unpack29(bool Solid)
       return;
   }
 
-  if (PPMError)
-    return;
-
   while (true)
   {
     UnpPtr&=MAXWINMASK;
@@ -217,7 +221,11 @@ void Unpack::Unpack29(bool Solid)
       int Ch=PPM.DecodeChar();
       if (Ch==-1)
       {
-        PPMError=true;
+        PPM.CleanUp();
+
+        // turn off PPM compression mode in case of error, so UnRAR will
+        // call PPM.DecodeInit in case it needs to turn it on back later.
+        UnpBlockType=BLOCK_LZ;
         break;
       }
       if (Ch==PPMEscChar)
@@ -254,6 +262,7 @@ void Unpack::Unpack29(bool Solid)
           }
           if (Failed)
             break;
+
           CopyString(Length+32,Distance+2);
           continue;
         }
@@ -402,7 +411,7 @@ bool Unpack::ReadEndOfBlock()
   else
   {
     NewFile=true;
-    NewTable=(BitField & 0x4000);
+    NewTable=(BitField & 0x4000)!=0;
     addbits(2);
   }
   TablesRead=!NewTable;
@@ -429,6 +438,8 @@ bool Unpack::ReadVMCode()
   Array<byte> VMCode(Length);
   for (int I=0;I<Length;I++)
   {
+    // Try to read the new buffer if only one byte is left.
+    // But if we read all bytes except the last, one byte is enough.
     if (InAddr>=ReadTop-1 && !UnpReadBuf() && I<Length-1)
       return(false);
     VMCode[I]=getbits()>>8;
@@ -491,30 +502,37 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
       FiltPos--;
   }
   else
-    FiltPos=LastFilter;
+    FiltPos=LastFilter; // use the same filter as last time
+
   if (FiltPos>Filters.Size() || FiltPos>OldFilterLengths.Size())
     return(false);
   LastFilter=FiltPos;
   bool NewFilter=(FiltPos==Filters.Size());
 
+  UnpackFilter *StackFilter=new UnpackFilter; // new filter for PrgStack
+
   UnpackFilter *Filter;
-  if (NewFilter)
+  if (NewFilter) // new filter code, never used before since VM reset
   {
+    // too many different filters, corrupt archive
+    if (FiltPos>1024)
+      return(false);
+
     Filters.Add(1);
     Filters[Filters.Size()-1]=Filter=new UnpackFilter;
+    StackFilter->ParentFilter=(uint)(Filters.Size()-1);
     OldFilterLengths.Add(1);
     Filter->ExecCount=0;
   }
-  else
+  else  // filter was used in the past
   {
     Filter=Filters[FiltPos];
+    StackFilter->ParentFilter=FiltPos;
     Filter->ExecCount++;
   }
 
-  UnpackFilter *StackFilter=new UnpackFilter;
-
   int EmptyCount=0;
-  for (int I=0;I<PrgStack.Size();I++)
+  for (uint I=0;I<PrgStack.Size();I++)
   {
     PrgStack[I-EmptyCount]=PrgStack[I];
     if (PrgStack[I]==NULL)
@@ -527,7 +545,7 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
     PrgStack.Add(1);
     EmptyCount=1;
   }
-  int StackPos=PrgStack.Size()-EmptyCount;
+  int StackPos=(int)(PrgStack.Size()-EmptyCount);
   PrgStack[StackPos]=StackFilter;
   StackFilter->ExecCount=Filter->ExecCount;
  
@@ -549,7 +567,8 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
   StackFilter->Prg.InitR[3]=VM_GLOBALMEMADDR;
   StackFilter->Prg.InitR[4]=StackFilter->BlockLength;
   StackFilter->Prg.InitR[5]=StackFilter->ExecCount;
-  if (FirstByte & 0x10)
+
+  if (FirstByte & 0x10)   // set registers to optional parameters if any
   {
     unsigned int InitMask=Inp.fgetbits()>>9;
     Inp.faddbits(7);
@@ -557,14 +576,17 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
       if (InitMask & (1<<I))
         StackFilter->Prg.InitR[I]=RarVM::ReadData(Inp);
   }
+
   if (NewFilter)
   {
     uint VMCodeSize=RarVM::ReadData(Inp);
     if (VMCodeSize>=0x10000 || VMCodeSize==0)
       return(false);
     Array<byte> VMCode(VMCodeSize);
-    for (int I=0;I<VMCodeSize;I++)
+    for (uint I=0;I<VMCodeSize;I++)
     {
+      if (Inp.Overflow(3))
+        return(false);
       VMCode[I]=Inp.fgetbits()>>8;
       Inp.faddbits(8);
     }
@@ -573,9 +595,10 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
   StackFilter->Prg.AltCmd=&Filter->Prg.Cmd[0];
   StackFilter->Prg.CmdCount=Filter->Prg.CmdCount;
 
-  int StaticDataSize=Filter->Prg.StaticData.Size();
+  size_t StaticDataSize=Filter->Prg.StaticData.Size();
   if (StaticDataSize>0 && StaticDataSize<VM_GLOBALMEMSIZE)
   {
+    // read statically defined data contained in DB commands
     StackFilter->Prg.StaticData.Add(StaticDataSize);
     memcpy(&StackFilter->Prg.StaticData[0],&Filter->Prg.StaticData[0],StaticDataSize);
   }
@@ -587,23 +610,27 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
   }
   byte *GlobalData=&StackFilter->Prg.GlobalData[0];
   for (int I=0;I<7;I++)
-    VM.SetValue((uint *)&GlobalData[I*4],StackFilter->Prg.InitR[I]);
-  VM.SetValue((uint *)&GlobalData[0x1c],StackFilter->BlockLength);
-  VM.SetValue((uint *)&GlobalData[0x20],0);
-  VM.SetValue((uint *)&GlobalData[0x2c],StackFilter->ExecCount);
+    VM.SetLowEndianValue((uint *)&GlobalData[I*4],StackFilter->Prg.InitR[I]);
+  VM.SetLowEndianValue((uint *)&GlobalData[0x1c],StackFilter->BlockLength);
+  VM.SetLowEndianValue((uint *)&GlobalData[0x20],0);
+  VM.SetLowEndianValue((uint *)&GlobalData[0x2c],StackFilter->ExecCount);
   memset(&GlobalData[0x30],0,16);
 
-  if (FirstByte & 8)
+  if (FirstByte & 8) // put data block passed as parameter if any
   {
-    uint DataSize=RarVM::ReadData(Inp);
-    if (DataSize>=0x10000)
+    if (Inp.Overflow(3))
       return(false);
-    unsigned int CurSize=StackFilter->Prg.GlobalData.Size();
+    uint DataSize=RarVM::ReadData(Inp);
+    if (DataSize>VM_GLOBALMEMSIZE-VM_FIXEDGLOBALSIZE)
+      return(false);
+    size_t CurSize=StackFilter->Prg.GlobalData.Size();
     if (CurSize<DataSize+VM_FIXEDGLOBALSIZE)
       StackFilter->Prg.GlobalData.Add(DataSize+VM_FIXEDGLOBALSIZE-CurSize);
     byte *GlobalData=&StackFilter->Prg.GlobalData[VM_FIXEDGLOBALSIZE];
-    for (int I=0;I<DataSize;I++)
+    for (uint I=0;I<DataSize;I++)
     {
+      if (Inp.Overflow(3))
+        return(false);
       GlobalData[I]=Inp.fgetbits()>>8;
       Inp.faddbits(8);
     }
@@ -614,11 +641,13 @@ bool Unpack::AddVMCode(unsigned int FirstByte,byte *Code,int CodeSize)
 
 bool Unpack::UnpReadBuf()
 {
-  int DataSize=ReadTop-InAddr;
+  int DataSize=ReadTop-InAddr; // Data left to process.
   if (DataSize<0)
     return(false);
   if (InAddr>BitInput::MAX_SIZE/2)
   {
+    // If we already processed more than half of buffer, let's move
+    // remaining data into beginning to free more space for new data.
     if (DataSize>0)
       memmove(InBuf,InBuf+InAddr,DataSize);
     InAddr=0;
@@ -638,8 +667,14 @@ void Unpack::UnpWriteBuf()
 {
   unsigned int WrittenBorder=WrPtr;
   unsigned int WriteSize=(UnpPtr-WrittenBorder)&MAXWINMASK;
-  for (int I=0;I<PrgStack.Size();I++)
+  for (size_t I=0;I<PrgStack.Size();I++)
   {
+    // Here we apply filters to data which we need to write.
+    // We always copy data to virtual machine memory before processing.
+    // We cannot process them just in place in Window buffer, because
+    // these data can be used for future string matches, so we must
+    // preserve them in original form.
+
     UnpackFilter *flt=PrgStack[I];
     if (flt==NULL)
       continue;
@@ -669,9 +704,29 @@ void Unpack::UnpWriteBuf()
           VM.SetMemory(0,Window+BlockStart,FirstPartLength);
           VM.SetMemory(FirstPartLength,Window,BlockEnd);
         }
+
+        VM_PreparedProgram *ParentPrg=&Filters[flt->ParentFilter]->Prg;
         VM_PreparedProgram *Prg=&flt->Prg;
+
+        if (ParentPrg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
+        {
+          // Copy global data from previous script execution if any.
+          Prg->GlobalData.Alloc(ParentPrg->GlobalData.Size());
+          memcpy(&Prg->GlobalData[VM_FIXEDGLOBALSIZE],&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],ParentPrg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
+        }
+
         ExecuteCode(Prg);
 
+        if (Prg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
+        {
+          // Save global data for next script execution.
+          if (ParentPrg->GlobalData.Size()<Prg->GlobalData.Size())
+            ParentPrg->GlobalData.Alloc(Prg->GlobalData.Size());
+          memcpy(&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],&Prg->GlobalData[VM_FIXEDGLOBALSIZE],Prg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
+        }
+        else
+          ParentPrg->GlobalData.Reset();
+
         byte *FilteredData=Prg->FilteredData;
         unsigned int FilteredDataSize=Prg->FilteredDataSize;
 
@@ -681,11 +736,35 @@ void Unpack::UnpWriteBuf()
         {
           UnpackFilter *NextFilter=PrgStack[I+1];
           if (NextFilter==NULL || NextFilter->BlockStart!=BlockStart ||
-              NextFilter->BlockLength!=FilteredDataSize)
+              NextFilter->BlockLength!=FilteredDataSize || NextFilter->NextWindow)
             break;
+
+          // Apply several filters to same data block.
+
           VM.SetMemory(0,FilteredData,FilteredDataSize);
-          VM_PreparedProgram *NextPrg=&PrgStack[I+1]->Prg;
+
+          VM_PreparedProgram *ParentPrg=&Filters[NextFilter->ParentFilter]->Prg;
+          VM_PreparedProgram *NextPrg=&NextFilter->Prg;
+
+          if (ParentPrg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
+          {
+            // Copy global data from previous script execution if any.
+            NextPrg->GlobalData.Alloc(ParentPrg->GlobalData.Size());
+            memcpy(&NextPrg->GlobalData[VM_FIXEDGLOBALSIZE],&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],ParentPrg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
+          }
+
           ExecuteCode(NextPrg);
+
+          if (NextPrg->GlobalData.Size()>VM_FIXEDGLOBALSIZE)
+          {
+            // Save global data for next script execution.
+            if (ParentPrg->GlobalData.Size()<NextPrg->GlobalData.Size())
+              ParentPrg->GlobalData.Alloc(NextPrg->GlobalData.Size());
+            memcpy(&ParentPrg->GlobalData[VM_FIXEDGLOBALSIZE],&NextPrg->GlobalData[VM_FIXEDGLOBALSIZE],NextPrg->GlobalData.Size()-VM_FIXEDGLOBALSIZE);
+          }
+          else
+            ParentPrg->GlobalData.Reset();
+
           FilteredData=NextPrg->FilteredData;
           FilteredDataSize=NextPrg->FilteredDataSize;
           I++;
@@ -700,7 +779,7 @@ void Unpack::UnpWriteBuf()
       }
       else
       {
-        for (int J=I;J<PrgStack.Size();J++)
+        for (size_t J=I;J<PrgStack.Size();J++)
         {
           UnpackFilter *flt=PrgStack[J];
           if (flt!=NULL && flt->NextWindow)
@@ -721,9 +800,9 @@ void Unpack::ExecuteCode(VM_PreparedProgram *Prg)
 {
   if (Prg->GlobalData.Size()>0)
   {
-    Prg->InitR[6]=int64to32(WrittenFileSize);
-    VM.SetValue((uint *)&Prg->GlobalData[0x24],int64to32(WrittenFileSize));
-    VM.SetValue((uint *)&Prg->GlobalData[0x28],int64to32(WrittenFileSize>>32));
+    Prg->InitR[6]=(uint)WrittenFileSize;
+    VM.SetLowEndianValue((uint *)&Prg->GlobalData[0x24],(uint)WrittenFileSize);
+    VM.SetLowEndianValue((uint *)&Prg->GlobalData[0x28],(uint)(WrittenFileSize>>32));
     VM.Execute(Prg);
   }
 }
@@ -735,7 +814,7 @@ void Unpack::UnpWriteArea(unsigned int StartPtr,unsigned int EndPtr)
     UnpSomeRead=true;
   if (EndPtr<StartPtr)
   {
-    UnpWriteData(&Window[StartPtr],-StartPtr & MAXWINMASK);
+    UnpWriteData(&Window[StartPtr],-(int)StartPtr & MAXWINMASK);
     UnpWriteData(Window,EndPtr);
     UnpAllBuf=true;
   }
@@ -744,14 +823,14 @@ void Unpack::UnpWriteArea(unsigned int StartPtr,unsigned int EndPtr)
 }
 
 
-void Unpack::UnpWriteData(byte *Data,int Size)
+void Unpack::UnpWriteData(byte *Data,size_t Size)
 {
   if (WrittenFileSize>=DestUnpSize)
     return;
-  int WriteSize=Size;
-  Int64 LeftToWrite=DestUnpSize-WrittenFileSize;
-  if (WriteSize>LeftToWrite)
-    WriteSize=int64to32(LeftToWrite);
+  size_t WriteSize=Size;
+  int64 LeftToWrite=DestUnpSize-WrittenFileSize;
+  if ((int64)WriteSize>LeftToWrite)
+    WriteSize=(size_t)LeftToWrite;
   UnpIO->UnpWrite(Data,WriteSize);
   WrittenFileSize+=Size;
 }
@@ -874,13 +953,18 @@ void Unpack::UnpInitData(int Solid)
     LastDist=LastLength=0;
 //    memset(Window,0,MAXWINSIZE);
     memset(UnpOldTable,0,sizeof(UnpOldTable));
+    memset(&LD,0,sizeof(LD));
+    memset(&DD,0,sizeof(DD));
+    memset(&LDD,0,sizeof(LDD));
+    memset(&RD,0,sizeof(RD));
+    memset(&BD,0,sizeof(BD));
     UnpPtr=WrPtr=0;
     PPMEscChar=2;
+    UnpBlockType=BLOCK_LZ;
 
     InitFilters();
   }
   InitBitInput();
-  PPMError=false;
   WrittenFileSize=0;
   ReadTop=0;
   ReadBorder=0;
@@ -895,10 +979,10 @@ void Unpack::InitFilters()
   OldFilterLengths.Reset();
   LastFilter=0;
 
-  for (int I=0;I<Filters.Size();I++)
+  for (size_t I=0;I<Filters.Size();I++)
     delete Filters[I];
   Filters.Reset();
-  for (int I=0;I<PrgStack.Size();I++)
+  for (size_t I=0;I<PrgStack.Size();I++)
     delete PrgStack[I];
   PrgStack.Reset();
 }
